import logging
import asyncio
from siridb.connector import SiriDBProtocol
from siridb.connector.lib.connection import SiriDBAsyncConnection


class PipeClient(SiriDBAsyncConnection):
    def __init__(self, pipe_name, loop=None):
        self._pipe_name = pipe_name
        self._protocol = None

    async def connect(self, username, password, dbname, loop=None):
        loop = loop or asyncio.get_event_loop()

        transport, self._protocol = await loop.create_unix_connection(
            path=self._pipe_name,
            protocol_factory=lambda: SiriDBProtocol(
                username, password, dbname))

        try:
            res = await self._protocol.auth_future
        except Exception as exc:
            logging.debug('Authentication failed: {}'.format(exc))
            transport.close()
            raise exc
        else:
            self._protocol.on_authenticated()



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


LENPOINTS = 36
DATA = {
    'series-001': [
        [1471254705000000005, 1.5],
        [1471254705000000007, -3.5],
        [1471254705000000010, -7.3]],
    'series-002': [
        [1471254705000000005, 5],
        [1471254705000000008, -3],
        [1471254705000000010, -7]],
    'series-003': [
        [1471254705000000005, 10.5],
        [1471254705000000007, -8.5],
        [1471254705000000010, -2.7]],
    'series-004': [
        [1471254705000000005, 6],
        [1471254705000000008, -8],
        [1471254705000000010, -9]],
    'linux-001': [
        [1471254705000000005, 7.3],
        [1471254705000000007, -6.4],
        [1471254705000000010, -9.8]],
    'linux-002': [
        [1471254705000000005, 2],
        [1471254705000000008, -7],
        [1471254705000000010, -9]],
    'linux-003': [
        [1471254705000000005, 2.9],
        [1471254705000000007, -5.7],
        [1471254705000000010, -0.3]],
    'linux-004': [
        [1471254705000000005, 3],
        [1471254705000000008, -9],
        [1471254705000000010, -8]],
    'windows-001': [
        [1471254705000000005, 9.3],
        [1471254705000000007, -3.3],
        [1471254705000000010, -1.6]],
    'windows-002': [
        [1471254705000000005, 4],
        [1471254705000000008, -8],
        [1471254705000000010, -2]],
    'windows-003': [
        [1471254705000000005, 4.3],
        [1471254705000000007, -7.9],
        [1471254705000000010, -1.2]],
    'windows-004': [
        [1471254705000000005, 2],
        [1471254705000000008, -5],
        [1471254705000000010, -7]],

}

TIME_PRECISION = 'ns'


class TestParenth(TestBase):
    title = 'Test parentheses'

    @default_test_setup(1, time_precision=TIME_PRECISION)
    async def run(self):
        await self.client0.connect()

        self.assertEqual(
            await self.client0.insert(DATA),
            {'success_msg': 'Successfully inserted {} point(s).'.format(
                LENPOINTS)})

        result = await self.client0.query('''
                list series
                    all - ("series-001" | "series-002" | /windows.*/)
               ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-003'],
                    ['series-004'],
                    ['linux-001'],
                    ['linux-002'],
                    ['linux-003'],
                    ['linux-004']
                ])
            }
        )

        result = await self.client0.query('''
                list series all - (
                    "series-001" | "series-002" | (/windows.*/ & /.*001/))
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-003'],
                    ['series-004'],
                    ['linux-001'],
                    ['linux-002'],
                    ['linux-003'],
                    ['linux-004'],
                    ['windows-002'],
                    ['windows-003'],
                    ['windows-004']
                ])
            }
        )
        result = await self.client0.query('''
                list series all - (
                    "series-001" | "series-002" | (/windows.*/ - /.*001/))
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-003'],
                    ['series-004'],
                    ['linux-001'],
                    ['linux-002'],
                    ['linux-003'],
                    ['linux-004'],
                    ['windows-001']
                ])
            }
        )

        result = await self.client0.query('''
                list series (
                    "series-001" | "series-002" | /windows.*/) - /.*003/
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['series-002'],
                    ['windows-001'],
                    ['windows-002'],
                    ['windows-004']
                ])
            }
        )

        result = await self.client0.query('''
                list series all - (/series.*/ ^ /.*001/)
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['linux-002'],
                    ['linux-003'],
                    ['linux-004'],
                    ['windows-002'],
                    ['windows-003'],
                    ['windows-004']
                ])
            }
        )

        self.assertEqual(
            await self.client0.query('''
                list series (/.*001/ & /linux.*/) - /.*001/
                '''),
            {
                'columns': ['name'],
                'series': []})

        result = await self.client0.query('''
                list series /.*001/ & (/series.*/ | /linux.*/)
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['linux-001']
                ])
            }
        )

        result = await self.client0.query('''
                list series /.*001/ & ((((/series.*/ | /linux.*/))))
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['linux-001']
                ])
            }
        )

        result = await self.client0.query('''
                list series (/.*001/ | /.*002/) & (/series.*/ | /linux.*/)
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['series-002'],
                    ['linux-001'],
                    ['linux-002']
                ])
            }
        )

        result = await self.client0.query('''
                list series ((/.*001/ | /.*002/) & (/series.*/ | /linux.*/))
                ''')
        result['series'] = sorted(result['series'])

        self.assertEqual(
            result,
            {
                'columns': ['name'],
                'series': sorted([
                    ['series-001'],
                    ['series-002'],
                    ['linux-001'],
                    ['linux-002']
                ])
            }
        )

        with self.assertRaisesRegex(
                QueryError,
                r'Query error at position 29. Expecting \*, all, '
                r'single_quote_str, double_quote_str or \('):
            await self.client0.query(
                'list series /.*/ - {}{}'.format('(' * 10, ')' * 10))

        with self.assertRaisesRegex(
                QueryError,
                'Memory allocation error or maximum recursion depth reached.'):
            await self.client0.query('''
                list series /.*/ -
                    {}/linux.*/{}'''.format('(' * 500, ')' * 500))

        await self.client0.query('alter database set list_limit 5000')
        with self.assertRaisesRegex(
                QueryError,
                'Limit must be a value between 1 and 5000 '
                'but received: 6000.*'):
            await self.client0.query(
                'list series limit 6000')

        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestParenth())



import asyncio
import functools
import random
import time
import string
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


TIME_PRECISION = 'ms'


class TestIntegerLoad(TestBase):
    title = 'Test inserts and response'

    GEN_POINTS = functools.partial(
        gen_points,
        tp=int,
        mi=-2**10,
        ma=2**10,
        n=5,
        time_precision=TIME_PRECISION,
        ts_gap=3)

    async def select(self, client, series, n):
        series = {s.name: s for s in series}
        chars = string.ascii_letters + string.digits
        for _ in range(n):
            regex = ''.join(random.choice(chars) for _ in range(3))
            res = await client.query(
                f'select max() prefix "max-", min() prefix "min-", '
                f'mean() prefix "mean-" from /.*{regex}.*/i')
            for s, p in res.items():
                self.assertEqual(sorted(series[s].points), p)
            await asyncio.sleep(0.2)

    async def insert(self, client, series, n):
        for _ in range(n):
            await client.insert_some_series(
                series, n=0.04, points=self.GEN_POINTS)
            await asyncio.sleep(0.2)

    @default_test_setup(
            1,
            time_precision=TIME_PRECISION,
            compression=True,
            auto_duration=True,
            optimize_interval=20)
    async def run(self):
        await self.client0.connect()

        series = gen_series(n=1000)

        insert = asyncio.ensure_future(
            self.insert(self.client0, series, n=50000))
        select = asyncio.ensure_future(
            self.select(self.client0, series, n=50000))
        await asyncio.gather(insert, select)

        self.client0.close()
        await asyncio.sleep(1800)


if __name__ == '__main__':
    random.seed(1)
    parse_args()
    run_test(TestIntegerLoad())



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


TIME_PRECISION = 's'


class TestExpiration(TestBase):
    title = 'Test shard expiration'

    GEN_POINTS = functools.partial(
        gen_points, n=1, time_precision=TIME_PRECISION)

    async def _test_series(self, client):

        result = await client.query('select * from "series float"')
        self.assertEqual(result['series float'], self.series_float)

        result = await client.query('select * from "series int"')
        self.assertEqual(result['series int'], self.series_int)

        result = await client.query(
            'list series name, length, type, start, end')
        result['series'].sort()
        self.assertEqual(
            result,
            {
                'columns': ['name', 'length', 'type', 'start', 'end'],
                'series': [
                    [
                        'series float',
                        10000, 'float',
                        self.series_float[0][0],
                        self.series_float[-1][0]],
                    [
                        'series int', 10000,
                        'integer',
                        self.series_int[0][0],
                        self.series_int[-1][0]],
                ]
            })

    async def insert(self, client, series, n, timeout=1):
        for _ in range(n):
            await client.insert_some_series(
                series, timeout=timeout, points=self.GEN_POINTS)
            await asyncio.sleep(1.0)

    @default_test_setup(
        2,
        time_precision=TIME_PRECISION,
        compression=True,
        optimize_interval=20)
    async def run(self):
        await self.client0.connect()

        await self.db.add_replica(self.server1, 0, sleep=30)
        # await self.db.add_pool(self.server1, sleep=30)

        await self.assertIsRunning(self.db, self.client0, timeout=30)

        await self.client1.connect()

        self.series_float = gen_points(
            tp=float, n=10000, time_precision=TIME_PRECISION, ts_gap='10m')
        random.shuffle(self.series_float)

        self.series_int = gen_points(
            tp=int, n=10000, time_precision=TIME_PRECISION, ts_gap='10m')
        random.shuffle(self.series_int)

        self.assertEqual(
            await self.client0.insert({
                'series float': self.series_float,
                'series int': self.series_int
            }), {'success_msg': 'Successfully inserted 20000 point(s).'})

        self.series_float.sort()
        self.series_int.sort()

        await self._test_series(self.client0)

        total = (await self.client0.query('count shards'))['shards']
        rest = (
            await self.client0.query('count shards where end > now - 3w')
        )['shards']

        self.assertGreater(total, rest)

        await self.client0.query('alter database set expiration_num 3w')
        await asyncio.sleep(50)  # wait for optimize to complete

        total = (await self.client0.query('count shards'))['shards']
        self.assertEqual(total, rest)

        await self.client0.query('alter database set expiration_log 2w')
        await self.client0.insert({
            'series_log': [
                [int(time.time()) - 3600*24*15, "expired_log"]
            ]
        })

        res = await self.client0.query('list series name, length "series_log"')
        self.assertEqual(len(res['series']), 0)

        await self.client0.insert({
            'series_log': [
                [int(time.time()) - 3600*24*15, "expired_log"],
                [int(time.time()) - 3600*24*7, "valid_log"],
            ]
        })

        res = await self.client0.query('list series name, length "series_log"')
        self.assertEqual(len(res['series']), 1)
        self.assertEqual(res['series'], [['series_log', 1]])

        await self.client0.query('alter database set drop_threshold 0.1')

        with self.assertRaisesRegex(
                QueryError,
                "This query would drop .*"):
            result = await self.client0.query(
                'alter database set expiration_num 1w')

        total = (await self.client0.query('count shards'))['shards']
        rest = (
            await self.client0.query('count shards where end > now - 1w')
        )['shards']

        result = await self.client0.query(
                'alter database set expiration_num 1w '
                'set ignore_threshold true')

        await asyncio.sleep(40)  # wait for optimize to complete

        total = (await self.client0.query('count shards'))['shards']
        self.assertEqual(total, rest)

        self.client0.close()
        self.client1.close()


if __name__ == '__main__':
    random.seed(1)
    parse_args()
    run_test(TestExpiration())



import asyncio
import logging
from .client import Client
from .client import InsertError
from .client import PoolError
from .client import QueryError
from .client import ServerError
from .client import UserAuthError
from .helpers import cleanup
from .helpers import gen_data
from .helpers import gen_points
from .helpers import gen_series
from .server import Server
from .siridb import SiriDB
from .testbase import default_test_setup
from .testbase import TestBase
from .series import Series
from .pipe_client import PipeClient as SiriDBAsyncUnixConnection
from .args import parse_args
from .task import Task


async def _run_test(test, loglevel):
    logger = logging.getLogger()
    logger.setLevel(loglevel)
    task = Task(test.title)

    try:
        await test.run()
    except Exception as e:
        task.stop(success=False)
        raise e
    else:
        task.stop(success=True)

    logger.setLevel('CRITICAL')

    await task.task


def run_test(test, loglevel='CRITICAL'):
    assert isinstance(test, TestBase)
    loop = asyncio.get_event_loop()
    cleanup()
    loop.run_until_complete(_run_test(test, loglevel))



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


TIME_PRECISION = 'ns'


class TestInsert(TestBase):
    title = 'Test inserts and response'

    GEN_POINTS = functools.partial(
        gen_points, n=1, time_precision=TIME_PRECISION)

    async def _test_series(self, client):

        result = await client.query('select * from "series float"')
        self.assertEqual(result['series float'], self.series_float)

        result = await client.query('select * from "series int"')
        self.assertEqual(result['series int'], self.series_int)

        result = await client.query(
            'list series name, length, type, start, end')
        result['series'].sort()
        self.assertEqual(
            result,
            {
                'columns': ['name', 'length', 'type', 'start', 'end'],
                'series': [
                    [
                        'series float',
                        10000, 'float',
                        self.series_float[0][0],
                        self.series_float[-1][0]],
                    [
                        'series int', 10000,
                        'integer',
                        self.series_int[0][0],
                        self.series_int[-1][0]],
                ]
            })

    async def insert(self, client, series, n, timeout=1):
        for _ in range(n):
            await client.insert_some_series(
                series, timeout=timeout, points=self.GEN_POINTS)
            await asyncio.sleep(1.0)

    @default_test_setup(2, time_precision=TIME_PRECISION, compression=False)
    async def run(self):
        await self.client0.connect()

        self.assertEqual(
            await self.client0.insert({}),
            {'success_msg': 'Successfully inserted 0 point(s).'})

        self.assertEqual(
            await self.client0.insert([]),
            {'success_msg': 'Successfully inserted 0 point(s).'})

        self.series_float = gen_points(
            tp=float, n=10000, time_precision=TIME_PRECISION, ts_gap='5m')
        random.shuffle(self.series_float)
        self.series_int = gen_points(
            tp=int, n=10000, time_precision=TIME_PRECISION, ts_gap='5m')
        random.shuffle(self.series_int)

        self.assertEqual(
            await self.client0.insert({
                'series float': self.series_float,
                'series int': self.series_int
            }), {'success_msg': 'Successfully inserted 20000 point(s).'})

        self.series_float.sort()
        self.series_int.sort()

        await self._test_series(self.client0)

        with self.assertRaises(InsertError):
            await self.client0.insert('[]')

        with self.assertRaises(InsertError):
            await self.client0.insert('[]')

        with self.assertRaises(InsertError):
            await self.client0.insert([{}])

        with self.assertRaises(InsertError):
            await self.client0.insert({'None': [[1, None]]})

        with self.assertRaises(InsertError):
            await self.client0.insert({'no points': []})

        with self.assertRaises(InsertError):
            await self.client0.insert({'no points': [[]]})

        self.assertEqual(
            await self.client0.insert({
                'ts_zero': [[0, 1]]
                }), {'success_msg': 'Successfully inserted 1 point(s).'})

        await self.client0.query('drop series "ts_zero"')

        with self.assertRaises(InsertError):
            await self.client0.insert([{'name': 'no points', 'points': []}])

        # timestamps should be interger values
        with self.assertRaises(InsertError):
            await self.client0.insert({'invalid ts': [[0.5, 6]]})

        # timestamps should be interger values
        with self.assertRaises(InsertError):
            await self.client0.insert(
                {'invalid ts': [[-1, 6]]})

        # empty series name is not allowed
        with self.assertRaises(InsertError):
            await self.client0.insert({'': [[1, 0]]})

        # empty series name is not allowed
        with self.assertRaises(InsertError):
            await self.client0.insert([{'name': '', 'points': [[1, 0]]}])

        await self.db.add_replica(self.server1, 0, sleep=30)
        # await self.db.add_pool(self.server1, sleep=30)

        await self.assertIsRunning(self.db, self.client0, timeout=30)

        await self.client1.connect()

        await self._test_series(self.client1)

        # Create some random series and start 25 insert task parallel
        series = gen_series(n=10000)
        tasks = [
            asyncio.ensure_future(
                self.client0.insert_some_series(
                    series,
                    timeout=0,
                    points=self.GEN_POINTS))
            for i in range(25)]
        await asyncio.gather(*tasks)

        await asyncio.sleep(2)

        # Check the result
        await self.assertSeries(self.client0, series)
        await self.assertSeries(self.client1, series)

        tasks = [
            asyncio.ensure_future(self.client0.query(
                    'drop series /.*/ set ignore_threshold true'))
            for i in range(5)]

        await asyncio.gather(*tasks)

        tasks = [
            asyncio.ensure_future(self.client0.query(
                    'drop shards set ignore_threshold true'))
            for i in range(5)]

        await asyncio.gather(*tasks)

        await asyncio.sleep(2)

        self.client0.close()
        self.client1.close()


if __name__ == '__main__':
    random.seed(1)
    parse_args()
    run_test(TestInsert())




NORMAL = '\x1B[0m'
RED = '\x1B[31m'
GREEN = '\x1B[32m'
YELLOW = '\x1B[33m'
LYELLOW = '\x1b[93m'


class Color:

    @staticmethod
    def success(text):
        return f'{GREEN}{text}{NORMAL}'

    @staticmethod
    def warning(text):
        return f'{YELLOW}{text}{NORMAL}'

    @staticmethod
    def error(text):
        return f'{RED}{text}{NORMAL}'

    @staticmethod
    def info(text):
        return f'{LYELLOW}{text}{NORMAL}'



#!/usr/bin/python3
import os
import sys
import argparse
import asyncio
import time
import logging
import string
import random
import datetime
import math
import collections
import signal
from siridb.connector import SiriDBClient


# Version information
__version_info__ = (0, 0, 5)
__version__ = '.'.join(map(str, __version_info__))
__maintainer__ = 'Jeroen van der Heijden'
__email__ = 'jeroen@cesbit.com'


# Logging definitions
_LOG_DATE_FMT = '%y%m%d %H:%M:%S'
_MAP_LOGLEVELS = {
    'DEBUG': logging.DEBUG,
    'INFO': logging.INFO,
    'WARNING': logging.WARNING,
    'ERROR': logging.ERROR,
    'CRITICAL': logging.CRITICAL
}

# counters etc.
total_processed = 0
total_failed = 0
start_time = time.time()

# Stop is used when pressing CTRL+C
stop = False


def setup_logger(args):
    # setup formatter without using colors
    formatter = logging.Formatter(
        fmt='[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] ' +
            '%(message)s',
        datefmt=_LOG_DATE_FMT,
        style='%')

    logger = logging.getLogger()

    logger.setLevel(_MAP_LOGLEVELS[args.log_level.upper()])

    if args.log_file_prefix:
        # create file handler
        ch = logging.RotatingFileHandler(
            args.log_file_prefix,
            maxBytes=args.log_file_max_size,
            backupCount=args.log_file_num_backups)

    else:
        # create console handler
        ch = logging.StreamHandler()

    # we can set the handler level to DEBUG since we control the root level
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(formatter)
    logger.addHandler(ch)


class Series:
    # Class values are set with the .init() method.
    _series = []
    _timestamp = None  # always in seconds
    _ts_factor = None
    _interval_range = None
    _r = None

    def __init__(self, r, allowed_kinds=(int, float, str), wrong_type=False):
        self.kind = r.choice(allowed_kinds)
        self.lval = {
            str: '',
            int: 0,
            float: 0.0
        }[self.kind]
        self.lts = self._timestamp

        factor = 10**r.randint(int(self.kind == int), 9)
        self.random_range = (
            int(r.random() * -factor),
            int(r.random() * factor) + 1)
        self.sign = 1

        self.ignore_last = r.random() > 0.95
        self.likely_equal = r.choice([0.01, 0.1, 0.2, 0.5, 0.99])
        self.likely_change_sign = r.choice([0.0, 0.1, 0.25, 0.5, 0.9])

        self.as_int = wrong_type and self.kind == float and r.random() > 0.9
        self.likely_inf = r.random() * 0.2 \
            if self.kind == float and r.random() > 0.95 else False
        self.likely_nan = r.random() * 0.2 \
            if self.kind == float and r.random() > 0.95 else False

        self.gen_float = wrong_type and self.kind == int and r.random() > 0.97

        self.name = self._gen_name()
        Series._series.append(self)

    def get_value(self):
        if self._r.random() < self.likely_change_sign:
            self.sign = -self.sign

        if self._r.random() > self.likely_equal:
            if self.kind == int:
                val = self.sign * self._r.randint(*self.random_range)
                if self.ignore_last:
                    self.lval = val
                else:
                    self.lval += val
                if self.lval.bit_length() > 63:
                    self.lval = 0

            elif self.kind == float:
                if self.likely_inf and self._r.random() < self.likely_inf:
                    return self.sign * math.inf
                elif self.likely_nan and self._r.random() < self.likely_nan:
                    return math.nan
                else:
                    val = self.sign * self._r.random() * self.random_range[1]
                    if self.ignore_last:
                        self.lval = val
                    else:
                        self.lval += val
                    if self.as_int:
                        self.lval = round(self.lval, 0)

        if self.gen_float:
            self.kind = float
            self.gen_float = False

        return self.lval

    @classmethod
    def init(cls, args, ts_factor):
        cls._r = random.Random()
        cls._r.seed(
            time.time() if args.seed_data is None else args.seed_data)

        series_rand = random.Random()
        series_rand.seed(
            time.time() if args.seed_series is None else args.seed_series)

        cls._ts_factor = ts_factor
        cls._timestamp = int(time.mktime(datetime.datetime.strptime(
            args.start_date,
            '%Y-%m-%d').timetuple()))

        n = math.ceil(args.ts_interval * 0.05)
        cls._interval_range = (-n, n)

        translate = {
            'int': int,
            'float': float,
            'str': str}
        kinds = [translate[k] for k in args.kinds]

        for i in range(args.num_series):
            Series(
                r=series_rand,
                allowed_kinds=kinds,
                wrong_type=args.wrong_type)

    def _gen_name(self):
        name = '/n:{}/range:{},{}/eq:{}/cs:{}/opt:{}{}{}{}{}'.format(
            len(self._series),
            self.random_range[0],
            self.random_range[1],
            self.likely_equal,
            self.likely_change_sign,
            '(ign_last)' if self.ignore_last else '',
            '(as_int)' if self.as_int else '',
            '(gen_float)' if self.gen_float else '',
            '(inf:{:.3f})'.format(self.likely_inf) if self.likely_inf else '',
            '(nan:{:.3f})'.format(self.likely_nan) if self.likely_nan else '')

        return name

    def get_ts(self, now, args):
        if args.ts_interval <= 0:
            return self._r.randint(self._timestamp, now) * self._ts_factor
        self.lts += args.ts_interval
        if args.ts_randomize:
            self.lts += self._r.randint(*self._interval_range)
        return self.lts * self._ts_factor

    @classmethod
    def pick_series(cls, n):
        series = cls._series[:]
        cls._r.shuffle(series)
        return series[:n]

    @classmethod
    def get_data(cls, args):
        nseries, npoints = args.series_per_batch, args.points_per_batch

        if nseries < 1:
            nseries = cls._r.randint(1, min(10000, cls.num()))

        if npoints < 1:
            npoints = cls._r.randint(nseries, 10000)

        series = cls.pick_series(nseries)
        now = int(time.time())
        data = collections.defaultdict(list)

        for s in series:
            val = s.get_value()
            ts = s.get_ts(now, args)
            data[s.name].append([ts, val])

        npoints -= nseries
        while npoints:
            s = cls._r.choice(series)
            ts = s.get_ts(now, args)
            val = s.get_value()
            data[s.name].append([ts, val])
            npoints -= 1

        return data

    @classmethod
    def num(cls):
        return len(cls._series)


async def get_ts_factor(siri):
    res = await siri.query('show time_precision')
    return 10**(['s', 'ms', 'us', 'ns'].index(res['data'][0]['value'])*3)


def queue_data(args):
    n = args.num_batches
    while n and stop is False:
        data = Series.get_data(args)
        yield data
        n -= 1


async def siridb_insert(siri, data, task_counter):
    '''Insert data into SiriDB.'''
    global total_processed
    global total_failed
    global start_time

    n = sum((len(p) for p in data.values()))
    start = time.time()
    try:
        await siri.insert(data)
        # await asyncio.sleep(0.1)
    except Exception as e:
        logging.exception(e)
        total_failed += n
    else:
        total_processed += n
        logging.info(
            'processed {} records, running {} task(s), {:d} records/sec'
            .format(
                total_processed,
                len(task_counter) - 1,
                int(total_processed // (time.time() - start_time))))
    finally:
        task_counter.pop()


async def dump_data(siri, args):
    task_counter = []

    try:
        await siri.connect()

        ts_factor = await get_ts_factor(siri)
        Series.init(args, ts_factor)

        q = queue_data(args)
        for data in q:

            task_counter.append(1)
            while len(task_counter) > args.max_parallel:
                await asyncio.sleep(0.2)

            asyncio.ensure_future(siridb_insert(siri, data, task_counter))

            # sleep 0 so the async loop will run to pick-up tasks
            await asyncio.sleep(args.sleep)

    except Exception as e:
        logging.exception(e)

    finally:
        while len(task_counter):
            await asyncio.sleep(0.5)
        siri.close()


def signal_handler(signal, frame):
    global stop
    logging.warning(
        'Asked to stop, existing data in the queue will be finished...')
    stop = True


if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-u', '--user',
        type=str,
        default='iris',
        help='database user')

    parser.add_argument(
        '-p', '--password',
        type=str,
        default='siri',
        help='password')

    parser.add_argument(
        '-d', '--dbname',
        type=str,
        default='dbtest',
        help='database name')

    parser.add_argument(
        '-s', '--servers',
        type=str,
        default='localhost:9000',
        help='siridb server(s)')

    parser.add_argument(
        '--seed-series',
        type=str,
        help='Optional seed for generating series. '
             'If no seed is given, the current timestamp will be used.')

    parser.add_argument(
        '--seed-data',
        type=str,
        help='Optional seed for generating data. '
             'If no seed is given, the current timestamp will be used.')

    parser.add_argument(
        '-v', '--version',
        action='store_true',
        help='print version information and exit')

    parser.add_argument(
        '--start-date',
        type=str,
        default='2017-01-01',
        help='Timestamps will be generated starting from this date. '
             '(using the format YYYY-MM-DD')

    parser.add_argument(
        '--ts-interval',
        type=int,
        default=60,
        help='Timestamp interval in seconds. '
             'When <= 0 a ramdom interval is used for each data point.')

    parser.add_argument(
        '--sleep',
        type=float,
        default=0.0,
        help='Sleep between inserts in seconds. (default: 0.0 seconds)')

    parser.add_argument(
        '--ts-randomize',
        action='store_true',
        help='A random value will be added to the timestamp intervals so '
             'they will be not exact anymore. In case --ts-interval is '
             '<= 0 this property will be ignored.')

    parser.add_argument(
        '--num-series',
        type=int,
        default=10000,
        help='number of series to generate')

    parser.add_argument(
        '--num-batches',
        type=int,
        default=10,
        help='number of batches to inserts. (value < 0 will be continues')

    parser.add_argument(
        '--series-per-batch',
        type=int,
        default=5000,
        help='number of series per batch. (0 for random)')

    parser.add_argument(
        '--points-per-batch',
        type=int,
        default=10000,
        help='number of points per batch. '
             '(must be equal or larger than series-per-batch or 0 for random)')

    parser.add_argument(
        '--kinds',
        nargs='+',
        default=('int', 'float'),
        choices=('int', 'float'))  # , 'str'

    parser.add_argument(
        '--wrong-type',
        action='store_true',
        help='Allow series to insert points using a wrong type')

    parser.add_argument(
        '--max-parallel',
        type=int,
        default=3,
        help='maximum number of allowed parallel inserts')

    parser.add_argument(
        '-l', '--log-level',
        default='info',
        help='set the log level (default: info)',
        choices=['debug', 'info', 'warning', 'error'])

    parser.add_argument(
        '--log-file-max-size',
        default=50000000,
        help='max size of log files before rollover ' +
        '(--log-file-prefix must be set)',
        type=int)

    parser.add_argument(
        '--log-file-num-backups',
        default=6,
        help='number of log files to keep (--log-file-prefix must be set)',
        type=int)

    parser.add_argument(
        '--log-file-prefix',
        help='path prefix for log files (when not provided we send the ' +
        'output to the console)',
        type=str)

    args = parser.parse_args()

    # respond to --version argument
    if args.version:
        sys.exit('''
SiriDB Data Generator Script {version}
Maintainer: {maintainer} <{email}>
Home-page: https://github.com/cesbit/siridb-email-check
        '''.strip().format(version=__version__,
                           maintainer=__maintainer__,
                           email=__email__))

    if args.num_series < args.series_per_batch:
        exit('num-series must be equal or greater than series-per-batch')

    if args.points_per_batch != 0 and \
            args.points_per_batch < args.series_per_batch:
        exit('points-per-batch must be equal or greater than series-per-batch')

    if args.max_parallel < 1:
        exit('max-parallel must be > 0')

    try:
        if datetime.datetime.strptime(
                args.start_date, '%Y-%m-%d') >= datetime.datetime.today():
            exit('start date must be a date before today.')
    except Exception:
        exit('invalid date: {}'.format(args.start_date))

    setup_logger(args)
    signal.signal(signal.SIGINT, signal_handler)

    siri = SiriDBClient(
        username=args.user,
        password=args.password,
        dbname=args.dbname,
        hostlist=[
            [s.strip() for s in server.split(':')]
            for server in args.servers.split(',')])

    loop = asyncio.get_event_loop()
    loop.run_until_complete(dump_data(siri, args))

    total_time = time.time() - start_time
    logging.info(
        'total time: {:.3f} seconds, '
        'processed: {}, '
        'failed: {}'
        .format(
            total_time,
            total_processed,
            total_failed))




import logging
import asyncio
import struct
import qpack
from siridb.connector import SiriDBProtocol
from siridb.connector.lib.connection import SiriDBAsyncConnection


class Package:

    __slots__ = ('pid', 'length', 'tipe', 'checkbit', 'data')

    struct_datapackage = struct.Struct('<IHBB')

    def __init__(self, barray):
        self.length, self.pid, self.tipe, self.checkbit = \
            self.__class__.struct_datapackage.unpack_from(barray, offset=0)
        self.length += self.__class__.struct_datapackage.size
        self.data = None

    def extract_data_from(self, barray):
        try:
            self.data = qpack.unpackb(
                barray[self.__class__.struct_datapackage.size:self.length],
                decode='utf-8')
        finally:
            del barray[:self.length]


class SiriDBServerProtocol(asyncio.Protocol):

    def __init__(self, on_package_received):
        self._buffered_data = bytearray()
        self._data_package = None
        self._on_package_received = on_package_received

    def data_received(self, data):
        '''
        override asyncio.Protocol
        '''
        self._buffered_data.extend(data)
        while self._buffered_data:
            size = len(self._buffered_data)
            if self._data_package is None:
                if size < Package.struct_datapackage.size:
                    return None
                self._data_package = Package(self._buffered_data)
            if size < self._data_package.length:
                return None
            try:
                self._data_package.extract_data_from(self._buffered_data)
            except KeyError as e:
                logging.error('Unsupported package received: {}'.format(e))
            except Exception as e:
                logging.exception(e)
                # empty the byte-array to recover from this error
                self._buffered_data.clear()
            else:
                self._on_package_received(self._data_package.data)
            self._data_package = None



import asyncio
import functools
import random
import time
import math
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


DATA = {
    'series-001 float': [
        [1471254705, 1.5],
        [1471254707, -3.5],
        [1471254710, -7.3]],
    'series-001 integer': [
        [1471254705, 5],
        [1471254708, -3],
        [1471254710, -7]],
    'series-002 float': [
        [1471254705, 3.5],
        [1471254707, -2.5],
        [1471254710, -8.3]],
    'series-002 integer': [
        [1471254705, 4],
        [1471254708, -1],
        [1471254710, -8]],
    'aggr': [
        [1447249033, 531], [1447249337, 534],
        [1447249633, 535], [1447249937, 531],
        [1447250249, 532], [1447250549, 537],
        [1447250868, 530], [1447251168, 520],
        [1447251449, 54], [1447251749, 54],
        [1447252049, 513], [1447252349, 537],
        [1447252649, 528], [1447252968, 531],
        [1447253244, 533], [1447253549, 538],
        [1447253849, 534], [1447254149, 532],
        [1447254449, 533], [1447254748, 537]],
    'huge': [
        [1471254705, 9223372036854775807],
        [1471254706, 9223372036854775806],
        [1471254707, 9223372036854775805],
        [1471254708, 9223372036854775804]],
    'equal ts': [
        [1471254705, 0], [1471254705, 1], [1471254705, 1],
        [1471254707, 0], [1471254707, 1], [1471254708, 0],
    ],
    'variance': [
        [1471254705, 2.75], [1471254706, 1.75], [1471254707, 1.25],
        [1471254708, 0.25], [1471254709, 0.5], [1471254710, 1.25],
        [1471254711, 3.5]
    ],
    'pvariance': [
        [1471254705, 0.0], [1471254706, 0.25], [1471254707, 0.25],
        [1471254708, 1.25], [1471254709, 1.5], [1471254710, 1.75],
        [1471254711, 2.75], [1471254712, 3.25]
    ],
    'filter': [
        [1471254705, 5],
        [1471254710, -3],
        [1471254715, -7],
        [1471254720, 7]
    ],
    'one': [
        [1471254710, 1]
    ],
    'log': [
        [1471254710, 'log line one'],
        [1471254712, 'log line two'],
        [1471254714, 'another line (three)'],
        [1471254716, 'and yet one more'],
    ],
    'special': [
        [1471254705, 0.1],
        [1471254706, math.nan],
        [1471254707, math.inf],
        [1471254708, -math.inf],
    ]
}


class TestTags(TestBase):
    title = 'Test tag and untag series'

    @default_test_setup(3, time_precision='s')
    async def run(self):
        await self.client0.connect()

        await self.client0.insert(DATA)

        res = await self.client0.query('''
            alter series /series.*/ tag `SERIES`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 4 series."})

        res = await self.client0.query('''
            alter series /.*/ tag `ALL`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 13 series."})

        res = await self.client0.query('''
            alter series /empty/ tag `EMPTY`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 0 series."})

        await asyncio.sleep(3.0)

        res = await self.client0.query('''
            alter series `ALL` - `SERIES` tag `OTHER`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 9 series."})

        await self.db.add_pool(self.server1)

        await asyncio.sleep(3.0)

        res = await self.client0.query('''
            alter series /series-00(1|2) integer/ tag `SERIES_INT`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 2 series."})

        res = await self.client0.query('''
            alter series 'one' untag `OTHER`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully untagged 1 series."})

        await self.assertIsRunning(self.db, self.client0, timeout=45)

        await asyncio.sleep(5)

        await self.db.add_replica(self.server2, 0)
        await asyncio.sleep(3.0)

        res = await self.client0.query('''
            alter series /series-00(1|2) float/ tag `SERIES_FLOAT`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 2 series."})

        res = await self.client0.query('''
            alter series 'huge' untag `OTHER`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully untagged 1 series."})

        await self.assertIsRunning(self.db, self.client0, timeout=45)

        res = await self.client0.query('''
            alter series 'one', 'huge', 'log' tag `SPECIAL`
        ''')
        self.assertEqual(
            res, {"success_msg": "Successfully tagged 3 series."})

        await self.client0.query('''
            alter series 'variance', 'pvariance' untag `OTHER`
        ''')

        await self.client0.query('''
            alter series `ALL` where type == float tag `F`
        ''')

        await self.client0.query('''
            alter series `ALL` tag `I`
        ''')

        await asyncio.sleep(3.0)

        await self.client0.query('''
            alter series `ALL` where type != integer untag `I`
        ''')

        await self.client1.connect()
        await self.client2.connect()

        for client in (self.client0, self.client1, self.client2):
            res = await self.client0.query('''
                list tags name, series
            ''')
            tags = sorted(res['tags'])
            self.assertEqual(tags, [
                ["ALL", 13],
                ["EMPTY", 0],
                ["F", 5],
                ["I", 7],
                ["OTHER", 5],
                ["SERIES", 4],
                ["SERIES_FLOAT", 2],
                ["SERIES_INT", 2],
                ["SPECIAL", 3],
            ])

        for series in ('huge', 'log', 'series-001 integer', 'one'):
            await self.client0.query('''
                drop series '{0}'
            '''.format(series))

        await asyncio.sleep(6.0)  # groups and tags are updates each 3 seconds

        for client in (self.client0, self.client1, self.client2):
            res = await self.client0.query('''
                list tags name, series
            ''')
            tags = sorted(res['tags'])
            self.assertEqual(tags, [
                ["ALL", 9],
                ["EMPTY", 0],
                ["F", 5],
                ["I", 4],
                ["OTHER", 4],
                ["SERIES", 3],
                ["SERIES_FLOAT", 2],
                ["SERIES_INT", 1],
                ["SPECIAL", 0],
            ])

        for tag in (
                'ALL',
                'EMPTY',
                'OTHER',
                'SERIES',
                'SERIES_FLOAT',
                'SERIES_INT',
                'SPECIAL'):
            await self.client0.query('''
                drop tag `{0}`
            '''.format(tag))

        await asyncio.sleep(1.5)

        for client in (self.client0, self.client1, self.client2):
            res = await client.query('''
                list tags name, series
            ''')
            tags = sorted(res['tags'])
            self.assertEqual(tags, [
                ["F", 5],
                ["I", 4],
            ])

        await self.client0.query('''
            alter tag `F` set name 'Float'
        ''')

        await asyncio.sleep(1.5)

        for client in (self.client0, self.client1, self.client2):
            res = await client.query('''
                list tags name, series
            ''')
            tags = sorted(res['tags'])
            self.assertEqual(tags, [
                ["Float", 5],
                ["I", 4],
            ])

        self.client2.close()
        self.client1.close()
        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestTags())



import sys
import time
import asyncio
from .spinner import Spinner
from .color import Color


class Task:
    def __init__(self, title):
        self.running = True
        self.task = asyncio.ensure_future(self.process())
        self.success = False
        self.title = title
        self.start = time.time()

    def stop(self, success):
        self.running = False
        self.success = success
        self.duration = time.time() - self.start

    async def process(self):
        spinner = Spinner()
        while self.running:
            sys.stdout.write(f'{self.title:.<76}{spinner.next}\r')
            sys.stdout.flush()
            await asyncio.sleep(0.2)

        if self.success:
            print(
                f'{self.title:.<76}'
                f'{Color.success("OK")} ({self.duration:.2f} seconds)')
        else:
            print(
                f'{self.title:.<76}'
                f'{Color.error("FAILED")} ({self.duration:.2f} seconds)')



import unittest
import time
import asyncio
import random
import math
import collections
from .siridb import SiriDB
from .server import Server
from .client import Client


def default_test_setup(nservers=1, **kwargs):
    def wrapper(func):
        async def wrapped(self):
            self.db = SiriDB(**kwargs)

            self.servers = [
                Server(n, title=self.title, **kwargs) for n in range(nservers)]
            for n, server in enumerate(self.servers):
                setattr(self, 'server{}'.format(n), server)
                setattr(self, 'client{}'.format(n), Client(self.db, server))
                server.create()
                await server.start()

            time.sleep(2.0)

            await self.db.create_on(self.server0, sleep=5)

            close = await func(self)

            if Server.TERMINAL is None or Server.HOLD_TERM is not True:
                for server in self.servers:
                    result = await server.stop()
                    self.assertTrue(
                        result,
                        msg='Server {} did not close correctly'.format(
                            server.name))

        return wrapped
    return wrapper


class TestBase(unittest.TestCase):

    title = 'No title set'

    async def run():
        raise NotImplementedError()

    async def assertIsRunning(self, db, client, timeout=None):
        while True:
            result = await client.query('list servers name, status')
            result = result['servers']
            try:
                assert len(result) == len(self.db.servers), \
                    'Server(s) are missing: {} (expecting: {})'.format(
                        result, self.db.servers)
            except AssertionError as e:
                if not timeout:
                    raise e
            else:
                try:
                    assert all([
                        status == 'running' for name, status in result]), \
                        'Not all servers have status running: {}'.format(
                            result)
                except AssertionError as e:
                    if not timeout:
                        raise e
                else:
                    break

            if timeout is not None:
                timeout -= 1

            await asyncio.sleep(1.0)

    async def assertSeries(self, client, series):
        d = {s.name: s.points for s in series if s.points}

        result = await client.query('list series name, length limit {}'.format(
            len(series)))
        result = {name: length for name, length in result['series']}
        for s in series:
            if s.points:
                length = result.get(s.name, None)
                assert length is not None, \
                    'series {!r} is missing in the result'.format(s.name)
                assert length == len(s.points) or \
                    s.commit_points() or \
                    length == len(s.points), \
                    'expected {} point(s) but found {} point(s) ' \
                    'for series {!r}' \
                    .format(len(s.points), length, s.name)

        n = min(len(series), 10)
        li = list(d.keys())
        random.shuffle(li)
        for i in range(n):
            result = await client.query('select * from "{}"'.format(li[i]))
            points = result[li[i]]
            expected = sorted(d[li[i]])
            assert len(points) == len(expected), \
                'incorrect number of points: {}, expected: {}'.format(
                    len(points), len(expected))
            c = collections.Counter([p[0] for p in points])
            for i, p in enumerate(points):
                ts, val = p
                if c[ts] == 1:
                    self.assertEqual(p, expected[i])

    def assertAlmostEqual(self, a, b, *args, **kwargs):
        if isinstance(b, dict):
            for series, points in b.items():
                assert isinstance(points, list), 'Expecting a list of points'
                for i, point in enumerate(points):
                    assert isinstance(point, list) and len(point) == 2, \
                        'Expecting a point to be a list of 2 items'
                    super().assertEqual(a[series][i][0], point[0])
                    if isinstance(a[series][i][1], str):
                        super().assertEqual(
                            a[series][i][1].replace(',', '.'),
                            point[1].replace(',', '.'))
                    elif math.isnan(a[series][i][1]):
                        assert math.isnan(point[1]), \
                            'Expecting point `{}` to be `nan`, got: `{}`' \
                            .format(i, point[1])
                    else:
                        super().assertAlmostEqual(
                            a[series][i][1],
                            point[1],
                            *args,
                            **kwargs)
        else:
            super().assertAlmostEqual(a, b, *args, **kwargs)



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


PI = 'ԉ'
Klingon = '     ' + \
    'qajunpaQHeylIjmo’ batlh DuSuvqang charghwI’ ‘It.'

data = {
    'string': [
        [1538660000, "some string value"],
        [1538660010, -123456789],
        [1538660020, -0.5],
        [1538660030, 1/3],
    ],
    'integer': [
        [1538660000, 1],
        [1538660010, 35.6],
        [1538660020, "-50%"],
        [1538660030, ""],
        [1538660035, "garbage"],
        [1538660040, "18446744073709551616"],
        [1538660050, "-18446744073709551616"],
    ],
    'double': [
        [1538660000, 1.0],
        [1538660010, -35],
        [1538660010, "-50%"],
        [1538660030, ""],
        [1538660035, "garbage"],
    ]
}

expected = {
    'string': [
        [1538660000, "some string value"],
        [1538660010, '-123456789'],
        [1538660020, '-0,500000'],
        [1538660030, '0,333333'],
    ],
    'integer': [
        [1538660000, 1],
        [1538660010, 35],
        [1538660020, -50],
        [1538660030, 0],
        [1538660035, 0],
        [1538660040, 9223372036854775807],
        [1538660050, -9223372036854775808],
    ],
    'double': [
        [1538660000, 1.0],
        [1538660010, -35.0],
        [1538660010, -50.0],
        [1538660030, 0.0],
        [1538660035, 0.0],
    ]
}


class TestSeries(TestBase):
    title = 'Test series object'

    @default_test_setup(1)
    async def run(self):
        await self.client0.connect()

        points = gen_points(n=10)

        self.assertEqual(
            await self.client0.insert({
                PI: points,
                Klingon: points
            }), {'success_msg': 'Successfully inserted 20 point(s).'})

        self.assertEqual(
            await self.client0.query('select * from "{}"'.format(PI)),
            {PI: sorted(points)})

        self.assertEqual(
            await self.client0.query('select * from "{}"'.format(Klingon)),
            {Klingon: sorted(points)})

        self.assertEqual(
            await self.client0.insert(data),
            {'success_msg': 'Successfully inserted 16 point(s).'})

        self.assertAlmostEqual(
            await self.client0.query(
                'select * from "string", "integer", "double"'),
            expected)

        self.assertAlmostEqual(
            await self.client0.query(
                'select * from "x", "string", "integer", "double", "nexist"'),
            expected)

        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestSeries())



#!/usr/bin/python3
import asyncio
import aiohttp
import argparse

URL = \
    'http://www.google.com/finance/getprices?' \
    'i={interval}&p={days}d&f=d,o,h,l,c,v&df=cpct&q={ticker}'


def _data_to_csv(data, ticker, interval):
    lines = []
    series = None
    ts = None
    tz_offset = None
    for n, line in enumerate(data.splitlines()):
        if line.startswith('COLUMNS='):
            series = [
                'GOOGLE-FINANCE-{}-{}'.format(ticker, column)
                for column in line[13:].split(',')]
            lines.append(',' + ','.join(series))
        elif line.startswith('TIMEZONE_OFFSET='):
            tz_offset = int(line[17:])
        elif tz_offset is not None:
            date, *fields = line.split(',')
            if date[0] == 'a':
                ts = int(date[1:])
                offset = 0
            else:
                offset = int(date) * interval
            lines.append('{},{:.2f},{:.2f},{:.2f},{:.2f},{}'.format(
                ts + offset, *map(float, fields[:-1]), fields[-1]))

    if series is None:
        raise Exception('Missing series in data')

    return '\n'.join(lines)


async def get_google_finance_data(ticker, interval, days):
    """Returns Google Finance Data in CSV format.

    :param ticker: This is the ticker symbol of the stock
    :param interval: Interval or frequency in seconds (minimal 60)
    :param days: The historical data period in days
    :return: string (csv format)
    """
    assert interval >= 60, \
        'Inteval lower than 60 is not supported by the Google Finance API.'

    async with aiohttp.ClientSession() as session:
        async with session.get(URL.format(
                interval=interval,
                days=days,
                ticker=ticker)) as resp:
            content = await resp.text()
            return _data_to_csv(content, ticker, interval)


async def save_google_finance_data(fn, ticker, interval, days):
    """Returns Google Finance Data in CSV format.

    :param fn: Filename where the csv data is saved.
    :param ticker: This is the ticker symbol of the stock
    :param interval: Interval or frequency in seconds (minimal 60)
    :param days: The historical data period in days
    :return: None
    """
    csv_data = await get_google_finance_data(ticker, interval, days)
    with open(fn, 'w') as f:
        f.write(csv_data)


async def print_google_finance_data(ticker, interval, days):
    """Prints Google Finance Data in CSV format.

    :param ticker: This is the ticker symbol of the stock
    :param interval: Interval or frequency in seconds (minimal 60)
    :param days: The historical data period in days
    :return: None
    """
    print(await get_google_finance_data(ticker, interval, days))

if __name__ == '__main__':
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-t', '--ticker',
        help='Ticker symbol of the stock',
        default='IBM',
        # required=True,
        type=str)

    parser.add_argument(
        '-i', '--interval',
        default=60,
        help='Interval or frequency in seconds',
        type=int)

    parser.add_argument(
        '-d', '--days',
        default=10,
        help='The historical data period in days',
        type=int)

    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(print_google_finance_data(
        ticker=args.ticker,
        interval=args.interval,
        days=args.days))



import asyncio
import functools
import random
import time
import math
import re
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


TIME_PRECISION = 's'
factor = 1


class TestAutoDuration(TestBase):
    title = 'Test auto duration'

    @default_test_setup(
        2,
        compression=True,
        buffer_size=1024,
        time_precision=TIME_PRECISION,
        auto_duration=True,
        optimize_interval=20)
    async def run(self):
        await self.client0.connect()

        await self.db.add_pool(self.server1, sleep=30)
        await self.assertIsRunning(self.db, self.client0, timeout=30)

        series = {}
        end_td = int(time.time())
        start_ts = end_td - (3600 * 24 * 7 * 10)
        tests = [[300, 10], [60, 5], [3600, 30], [60, 90], [10, 1]]

        for i, cfg in enumerate(tests):
            interval, r = cfg
            for nameval in [['int', 42], ['float', 3.14], ['str', 'hi']]:
                name, val = nameval
                series['{}-{}'.format(name, i)] = [
                    [(t + random.randint(-r, r)) * factor, val]
                    for t in range(start_ts, end_td, interval)
                ]

        self.assertEqual(
            await self.client0.insert(series),
            {'success_msg': 'Successfully inserted {} point(s).'.format(
                2484720)})

        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestAutoDuration())



#!/usr/bin/env python
import struct
import argparse
import asyncio
import qpack


class TeeServerProtocol(asyncio.Protocol):
    def connection_made(self, transport):
        peername = transport.get_extra_info('peername')
        print('Connection from {}'.format(peername))
        self.transport = transport

    def data_received(self, data):
        print('Data received: {}'.format(len(data)))
        header = struct.unpack_from('<LHBB', data)
        print(header)
        # self.transport.write(b'test')


async def main(args):
    loop = asyncio.get_running_loop()

    server = await loop.create_server(
        lambda: TeeServerProtocol(),
        '0.0.0.0',
        args.port)

    async with server:
        await server.serve_forever()


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", "--port", type=int, default=9104)
    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(args))



import asyncio
import functools
import os
import subprocess
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


LENPOINTS = 12
DATA = {
    'series-001': [
        [1471254705000000005, 1.5],
        [1471254705000000007, -3.5],
        [1471254705000000010, -7.3]],
    'series-002': [
        [1471254705000000005, 5],
        [1471254705000000008, -3],
        [1471254705000000010, -7]],
    'series-003': [
        [1471254705000000005, 10.5],
        [1471254705000000007, -8.5],
        [1471254705000000010, -2.7]],
    'series-004': [
        [1471254705000000005, 6],
        [1471254705000000008, -8],
        [1471254705000000010, -9]],
}

TIME_PRECISION = 'ns'


class TestCreateDatabase(TestBase):
    title = 'Test create database'

    @default_test_setup(2, time_precision=TIME_PRECISION)
    async def run(self):
        await self.client0.connect()

        tasks = [
            asyncio.ensure_future(
                SiriDB(
                    dbname="db_{}".format(i),
                    time_precision=TIME_PRECISION).create_on(
                        server=self.server0
                        ))
            for i in range(30)]

        await asyncio.gather(*tasks)

        self.assertEqual(
            await self.client0.insert(DATA),
            {'success_msg': 'Successfully inserted {} point(s).'.format(
                LENPOINTS)})

        self.assertEqual(
            await self.client0.query('create group `b` for /series.*/'),
            {'success_msg': "Successfully created group 'b'."})

        time.sleep(3)

        self.client0.close()

        for i in range(30):
            client = Client(
                db=SiriDB(dbname="db_{}".format(i)),
                servers=self.servers)
            await client.connect()
            self.assertEqual(
                await client.insert(DATA),
                {'success_msg': 'Successfully inserted {} point(s).'.format(
                    LENPOINTS)})
            client.close()

        await self.client0.connect()

        self.assertEqual(
            await self.client0.query(
                'select max(1) from /series.*/ '
                'merge as "max" using max(1)'),
            {'max': [
                [1471254705000000005, 10.5],
                [1471254705000000007, -3.5],
                [1471254705000000008, -3.0],
                [1471254705000000010, -2.7]
            ]})

        await self.db.add_pool(self.server1)
        await self.assertIsRunning(self.db, self.client0, timeout=20)

        await asyncio.sleep(45)

        await SiriDB(dbname="dbtest").drop_db(server=self.server1)

        tasks = [
            asyncio.ensure_future(
                SiriDB(
                    dbname="db_{}".format(i)).drop_db(
                        server=self.server0
                        ))
            for i in range(10)]

        await asyncio.gather(*tasks)

        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestCreateDatabase())



import requests
import json
from testing import gen_points
import asyncio
import functools
import random
import time
import math
import re
import qpack
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args

TIME_PRECISION = 's'


class TestHTTPAPI(TestBase):
    title = 'Test HTTP API requests'

    @default_test_setup(3, time_precision=TIME_PRECISION)
    async def run(self):
        await self.client0.connect()

        x = requests.get(
            f'http://localhost:9020/get-version', auth=('sa', 'siri'))

        self.assertEqual(x.status_code, 200)
        v = x.json()
        self.assertTrue(isinstance(v, list))
        self.assertTrue(isinstance(v[0], str))

        x = requests.post(
            f'http://localhost:9020/insert/dbtest',
            auth=('iris', 'siri'),
            headers={'Content-Type': 'application/json'})

        self.assertEqual(x.status_code, 400)

        series_float = gen_points(
            tp=float, n=10000, time_precision=TIME_PRECISION, ts_gap='5m')

        series_int = gen_points(
            tp=int, n=10000, time_precision=TIME_PRECISION, ts_gap='5m')

        data = {
            'my_float': series_float,
            'my_int': series_int
        }

        x = requests.post(
            f'http://localhost:9020/insert/dbtest',
            data=json.dumps(data),
            auth=('iris', 'siri'),
            headers={'Content-Type': 'application/json'}
        )

        self.assertEqual(x.status_code, 200)
        self.assertDictEqual(x.json(), {
            'success_msg': 'Successfully inserted 20000 point(s).'})

        data = {
            'dbname': 'dbtest',
            'host': 'localhost',
            'port': 9000,
            'username': 'iris',
            'password': 'siri'
        }

        x = requests.post(
            f'http://localhost:9021/new-pool',
            data=json.dumps(data),
            auth=('sa', 'siri'),
            headers={'Content-Type': 'application/json'})

        self.assertEqual(x.status_code, 200)
        self.assertEqual(x.json(), 'OK')

        self.db.servers.append(self.server1)
        await self.assertIsRunning(self.db, self.client0, timeout=30)

        data = {'data': [[1579521271, 10], [1579521573, 20]]}
        x = requests.post(
            f'http://localhost:9020/insert/dbtest',
            json=data,
            auth=('iris', 'siri'))

        self.assertEqual(x.status_code, 200)
        self.assertDictEqual(x.json(), {
            'success_msg': 'Successfully inserted 2 point(s).'})

        x = requests.post(
            f'http://localhost:9020/query/dbtest',
            json={'q': 'select * from "data"'},
            auth=('iris', 'siri'))

        self.assertEqual(x.status_code, 200)
        self.assertEqual(x.json(), data)

        x = requests.post(
            f'http://localhost:9020/query/dbtest',
            json={'q': 'select * from "data"', 't': 'ms'},
            auth=('iris', 'siri'))

        data = {
            'data': [[p[0] * 1000, p[1]] for p in data['data']]
        }

        self.assertEqual(x.status_code, 200)
        self.assertEqual(x.json(), data)

        x = requests.post(
            f'http://localhost:9020/query/dbtest',
            data=qpack.packb({
                'q': 'select sum(1579600000) from "data"',
                't': 'ms'}),
            headers={'Content-Type': 'application/qpack'},
            auth=('iris', 'siri'))

        self.assertEqual(x.status_code, 200)
        self.assertEqual(
            qpack.unpackb(x.content, decode='utf8'),
            {'data': [[1579600000000, 30]]})

        x = requests.post(
            f'http://localhost:9021/new-account',
            json={'account': 't', 'password': ''},
            auth=('sa', 'siri'))

        self.assertEqual(x.status_code, 400)
        self.assertEqual(x.json(), {
            'error_msg':
                'service account name should have at least 2 characters'})

        x = requests.post(
            f'http://localhost:9021/new-account',
            json={'account': 'tt', 'password': 'pass'},
            auth=('sa', 'siri'))

        self.assertEqual(x.status_code, 200)

        data = {
            'dbname': 'dbtest',
            'host': 'localhost',
            'port': 1234,
            'pool': 0,
            'username': 'iris',
            'password': 'siri'
        }

        auth = ('tt', 'pass')
        x = requests.post(
            f'http://localhost:9021/new-replica', json=data, auth=auth)

        self.assertEqual(x.status_code, 400)
        self.assertEqual(x.json(), {
            'error_msg': "database name already exists: 'dbtest'"})

        x = requests.post(
            f'http://localhost:9022/new-replica', json=data, auth=auth)
        self.assertEqual(x.status_code, 401)

        auth = ('sa', 'siri')
        x = requests.post(
            f'http://localhost:9022/new-replica', json=data, auth=auth)

        self.assertEqual(x.status_code, 400)
        self.assertEqual(x.json(), {
            'error_msg':
                "connecting to server 'localhost:1234' failed with error: "
                "connection refused"})

        data['port'] = 9000
        x = requests.post(
            f'http://localhost:9022/new-replica', json=data, auth=auth)
        self.assertEqual(x.status_code, 200)
        self.assertEqual(x.json(), 'OK')

        self.db.servers.append(self.server2)
        await self.assertIsRunning(self.db, self.client0, timeout=50)

        x = requests.get(
            f'http://localhost:9022/get-databases', auth=auth)
        self.assertEqual(x.status_code, 200)
        self.assertEqual(x.json(), ['dbtest'])

        self.client0.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestHTTPAPI())



#!/usr/bin/python3
from testing import run_test
from testing import Server
from testing import parse_args
from test_auto_duration import TestAutoDuration
from test_buffer import TestBuffer
from test_compression import TestCompression
from test_create_database import TestCreateDatabase
from test_expiration import TestExpiration
# from test_grammar import TestGrammar
from test_group import TestGroup
from test_http_api import TestHTTPAPI
from test_insert import TestInsert
from test_list import TestList
from test_log import TestLog
from test_parentheses import TestParenth
from test_pipe_support import TestPipeSupport
from test_pool import TestPool
from test_select import TestSelect
from test_select_ns import TestSelectNano
from test_series import TestSeries
from test_server import TestServer
from test_tags import TestTags
from test_user import TestUser


if __name__ == '__main__':
    parse_args()
    run_test(TestAutoDuration())
    run_test(TestBuffer())
    run_test(TestCompression())
    run_test(TestCreateDatabase())
    run_test(TestExpiration())
    # run_test(TestGrammar())
    run_test(TestGroup())
    run_test(TestHTTPAPI())
    run_test(TestInsert())
    run_test(TestList())
    run_test(TestLog())
    run_test(TestParenth())
    run_test(TestPipeSupport())
    run_test(TestPool())
    run_test(TestSelect())
    run_test(TestSelectNano())
    run_test(TestSeries())
    run_test(TestServer())
    run_test(TestTags())
    run_test(TestUser())



k_map = {
    'r_doubleq_str': {
        'k_as': '"MERGED"',
        'k_suffix': '"SUFFIX"',
        'k_prefix': '"PREFIX"',
        'series_name': '"000000"',
        'k_filter': 10,  # '"10"',
        'uuid': '"koos-VirtualBox:9010"',
        'k_name': '"000000"',
        'k_user': '"USER"',
        'k_password': '"PASSWORD"',
        'k_status': '"running"',
        'k_expression': '"/.*/"',
        'k_address': '"localhost"',
        'k_buffer_path': '"BUFFER_PATH"',
        'k_dbpath': '"DBPATH"',
        'k_uuid': '"UUID"',
        'k_version': '"VERSION"',
        'k_reindex_progress': '"REINDEX_PROGRESS"',
        'k_sync_progress': '"SYNC_PROGRESS"',
        'k_timezone': '"NAIVE"',
        'k_ip_support': '"ALL"',
        'k_libuv': '"1.8.0"',
        'k_server': '"SERVER"',
	    'k_tee_pipe_name': '"PIPENAME"',
        'k_shard_duration': 86400,
        'k_expiration_num': 0,
        'k_expiration_log': 0,

        'aggregate_functions': '"1970-1-1 1:00:10"',
	    'k_start': '"1970-1-1 1:00:00"',
        'k_after': '"1970-1-1 1:00:00"',
        'k_between': '"1970-1-1 1:00:00"',
        'k_before': '"1970-1-1 1:01:00"',
        'k_and': '"1970-1-1 1:01:00"',
        'k_end': '"1970-1-1 1:01:00"',
        },
    'r_integer': {
        'k_series': 0,  # GROUPS
        'k_active_handles': 0,  # SERVERS
        'k_buffer_size': 0,  # SERVERS
        'k_port': 9000,  # SERVERS
        'k_startup_time': 0,  # SERVERS
        'k_max_open_files': 0,  # SERVERS
        'k_mem_usage': 0,  # SERVERS
        'k_open_files': 0,  # SERVERS
        'k_received_points': 0,  # SERVERS
        'k_uptime': 0,  # SERVERS,
        'k_servers': 0,  # POOLS
        'k_limit': 10,
        'k_sid': 0,
        'k_pool': 0,
        'k_filter': 10,
        'k_size': 10,
        'k_length': 10,
        'aggregate_functions': 10,
        'k_start': 0,
        'k_after': 0,
        'k_between': 0,
        'k_before': 60,
        'k_and': 60,
        'k_end': 60,
        'k_shard_duration': 86400,
        'k_expiration_num': 0,
        'k_expiration_log': 0,
    },
    'r_float': {
        'k_filter': 10.0,
        'k_drop_threshold': 0.99},
    'r_time_str': {
        'aggregate_functions': '10s',
        'k_start': '0d',
        'k_after': '0d',
        'k_between': '0d',
        'k_before': '1m',
        'k_and': '1m',
        'k_end': '1m',
        'k_shard_duration': '1d',
        'k_expiration_num': '0d',
        'k_expiration_log': '0d',
    },
    'r_uuid_str': {
        'r_uuid_str': '"UUID"'},
    'r_uinteger': {
    },
    'r_grave_str': {
	    'group_name': '`GROUP`',
        'tag_name': '`TAG`',
        'group_tag_match': '`GROUP_OR_TAG`',
    },
    'r_regex': {
        'r_regex': '/.*/'
    },
    'r_comment': {
        'r_comment': '#',
    },
}



SPINNER1 = \
    ('▁', '▂', '▃', '▄', '▅', '▆', '▇', '█', '▇', '▆', '▅', '▄', '▃', '▁')
SPINNER2 = \
    ('⠁', '⠂', '⠄', '⡀', '⢀', '⠠', '⠐', '⠈')
SPINNER3 = \
    ('◐', '◓', '◑', '◒')


class Spinner:

    def __init__(self, charset=SPINNER3):
        self._idx = 0
        self._charset = charset
        self._len = len(charset)

    @property
    def next(self):
        char = self._charset[self._idx]
        self._idx += 1
        self._idx %= self._len
        return char



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


DATA = {
    'a1': [[0, 0]],
    'a2': [[0, 0]],
    'b1': [[0, 0]],
    'b2': [[0, 0]],
    'c1': [[0, 0]],
    'c2': [[0, 0]]
}


class TestGroup(TestBase):
    title = 'Test group object'

    @default_test_setup(2)
    async def run(self):
        await self.client0.connect()

        with self.assertRaisesRegex(
                QueryError,
                'Group name should be at least 1 characters.'):
            await self.client0.query('create group `` for /c.*/')

        with self.assertRaisesRegex(
                QueryError,
                'Group name should be at least 1 characters.'):
            await self.client0.query('create group `` for /c.*/')

        with self.assertRaisesRegex(
                QueryError,
                'Group name should be at most [0-9]+ characters.'):
            await self.client0.query(
                'create group `{}` for /c.*/'.format('a' * 300))

        self.assertEqual(
            await self.client0.query('create group `a` for /a.*/'),
            {'success_msg': "Successfully created group 'a'."})

        with self.assertRaisesRegex(
                QueryError,
                'Group \'a\' already exists.'):
            await self.client0.query('create group `a` for /a.*/')

        self.assertEqual(
            await self.client0.insert(DATA),
            {'success_msg': 'Successfully inserted 6 point(s).'})

        time.sleep(3)

        self.assertEqual(
            await self.client0.query('count series `a`'),
            {'series': 2})

        self.assertEqual(
            await self.client0.query('create group `b` for /b.*/'),
            {'success_msg': "Successfully created group 'b'."})

        time.sleep(3)

        self.assertEqual(
            await self.client0.query('count series `b`'),
            {'series': 2})

        await self.db.add_pool(self.server1)
        await self.assertIsRunning(self.db, self.client0, timeout=60)
        await self.client1.connect()

        self.assertEqual(
            await self.client1.query('create group `c` for /c.*/'),
            {'success_msg': "Successfully created group 'c'."})

        time.sleep(3)

        result = await self.client1.query('list groups series')
        self.assertEqual(result.pop('groups'), [[2], [2], [2]])

        with self.assertRaisesRegex(
                QueryError,
                "Cannot compile regular expression.*"):
            result = await self.client1.query('create group `invalid` for /(/')

        self.assertEqual(
            await self.client1.query('create group `one` for /.1/'),
            {'success_msg': "Successfully created group 'one'."})

        self.assertEqual(
            await self.client1.query('alter group `one` set name "two"'),
            {'success_msg': "Successfully updated group 'two'."})

        self.assertEqual(
            await self.client1.query('alter group `two` set expression /.2/'),
            {'success_msg': "Successfully updated group 'two'."})

        time.sleep(3)

        result = await self.client0.query('list series `a` & `two`')
        self.assertEqual(sorted(result.pop('series')), [['a2']])

        result = await self.client0.query('list series `a` | `two`')
        self.assertEqual(
            sorted(result.pop('series')),
            [['a1'], ['a2'], ['b2'], ['c2']])

        result = await self.client0.query('list series `a` ^ `two`')
        self.assertEqual(
            sorted(result.pop('series')),
            [['a1'], ['b2'], ['c2']])

        result = await self.client0.query('list series `a` - `two`')
        self.assertEqual(sorted(result.pop('series')), [['a1']])

        result = await self.client0.query('list series `a`, `two` - "c2"')
        self.assertEqual(
            sorted(result.pop('series')),
            [['a1'], ['a2'], ['b2']])

        result = await self.client0.query('list series `a`, `two` & "c2"')
        self.assertEqual(sorted(result.pop('series')), [['c2']])

        with self.assertRaisesRegex(
                QueryError,
                "Cannot compile regular expression.*"):
            await self.client1.query('alter group `a` set expression /(.*/')

        self.assertEqual(
            await self.client1.query('alter group `a` set expression /b.*/'),
            {'success_msg': "Successfully updated group 'a'."})

        # await self.client0.query('drop group `a`')
        # await self.client0.query('drop group `b`')
        await self.client0.query('drop group `c`')

        with self.assertRaisesRegex(
                QueryError,
                'Group \'c\' does not exist.'):
            await self.client0.query('drop group `c`')

        await self.client0.query('create group `all` for /.*/ # bla')

        await self.client0.query('alter group `all` set expression /.*/ # bla')

        self.assertEqual(
            await self.client0.query('count groups'),
            {'groups': 4})

        await asyncio.sleep(2)

        self.assertEqual(
            await self.client0.query('count groups where series > 2'),
            {'groups': 2})

        self.client0.close()
        self.client1.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestGroup())



import asyncio
import functools
import random
import time
from testing import Client
from testing import default_test_setup
from testing import gen_data
from testing import gen_points
from testing import gen_series
from testing import InsertError
from testing import PoolError
from testing import QueryError
from testing import run_test
from testing import Series
from testing import Server
from testing import ServerError
from testing import SiriDB
from testing import TestBase
from testing import UserAuthError
from testing import parse_args


class TestPool(TestBase):
    title = 'Test pool object'

    async def insert(self, client, series, n, timeout=1):
        for _ in range(n):
            await client.insert_some_series(series, n=0.01, timeout=timeout)
            await asyncio.sleep(1.0)

    @default_test_setup(4)
    async def run(self):

        series = gen_series(n=10000)

        await self.client0.connect()

        task0 = asyncio.ensure_future(self.insert(
            self.client0,
            series,
            200))

        await asyncio.sleep(5)

        await self.db.add_pool(self.server1, sleep=3)

        with self.assertRaises(AssertionError):
            await self.db.add_pool(self.server2, sleep=3)

        await self.client1.connect()
        task1 = asyncio.ensure_future(self.insert(
            self.client1,
            series,
            150))

        await asyncio.sleep(5)

        await self.assertIsRunning(self.db, self.client0, timeout=200)

        await asyncio.sleep(25)

        await self.db.add_replica(self.server3, 1, sleep=3)
        await self.client3.connect()
        await self.assertIsRunning(self.db, self.client3, timeout=200)

        await asyncio.sleep(30)

        await self.db.add_pool(self.server2, sleep=3)
        await self.client2.connect()

        task2 = asyncio.ensure_future(self.insert(
            self.client2,
            series,
            100))

        await self.assertIsRunning(self.db, self.client0, timeout=600)

        await asyncio.wait_for(task0, None)
        await asyncio.wait_for(task1, None)
        await asyncio.wait_for(task2, None)

        await asyncio.sleep(2)

        await self.assertSeries(self.client0, series)
        await self.assertSeries(self.client1, series)
        await self.assertSeries(self.client2, series)
        await self.assertSeries(self.client3, series)

        self.client0.close()
        self.client1.close()
        self.client2.close()
        self.client3.close()


if __name__ == '__main__':
    parse_args()
    run_test(TestPool())
